/* since linux doesn't have a proper getloadavg() fn, we make it out of the 
	entry in /proc
*/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "yaunc.h"

#ifndef MAXLINE
 #define MAXLINE 1024
#endif

int
getloadavg(double load[], int sample)
{
	FILE * fd;
	char buffer[MAXLINE];
	int i;

	if ( (fd = fopen("/proc/loadavg", "r")) == NULL )
		return -1;

	fgets(buffer, MAXLINE, fd);

	for (i = 0; i < sample; i++)
	{
		errno = 0;
		load[i] = strtod(buffer, NULL);
		if ( errno == ERANGE )
			return -1;
	}

	if ( fclose(fd) != 0)
			err_ret("%s: fclose(/proc/loadavg) error", NAME);

	return 0;
}

/* make use of /proc/uptime and the fact that it stores total idle time */
time_t
get_uptime(float * idle)
{
	FILE * fd;
	float uptime;

	if ( (fd = fopen("/proc/uptime", "r")) == NULL ) {
		err_ret("%s: Failed to open /proc/uptime", NAME);
		return -1;
	}

	if ( fscanf(fd, "%f %f", &uptime, idle) != 2) {
		err_ret("%s: fscanf on /proc/uptime failed", NAME);
		if ( fclose(fd) != 0)
			err_ret("%s: fclose(/proc/uptime) error", NAME);
		return -1;
	}	
	/* as per the recommendation of the project maintainer - somehow this
		computes idle time... I'm not sure how we'll do it on nonlinux
		OS's yet...
	*/
	*idle = (*idle / uptime);

	/* clean things up */
	if ( fclose(fd) != 0) 
			err_ret("%s: fclose(/proc/uptime) error", NAME);

	/* minutes, not seconds as per protocol */
	return (time_t) (uptime / 60);
}
