#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>

#include "yaunc.h"

char *
get_dns(const char * hostname)
{
	int n = 1;
	struct in_addr in;			/* find a better way later */
	struct hostent * dns_data;

	dns_data = gethostbyname(hostname);

	/* keep trying to do the dns lookup so long as we are getting TRY_AGAIN
		errors.  Otherwise, we'll exit
	*/	
	while (h_errno == TRY_AGAIN) {
		if (n >= 256) {
			err_ret("%s: dns lookup exceed maximum allowed time", NAME);
			return NULL;
		}	
		/* sleep for n seconds */
		while (sleep(n) != 0)
		   ;
		dns_data = gethostbyname(hostname);
		n = n * 2;
	}	

	/* some other error occurred */
	if (dns_data == NULL) {
		switch (h_errno) {
			case NETDB_INTERNAL: 
				err_ret("%s: internal netdb library error", NAME);
				break;
			case HOST_NOT_FOUND:
				err_ret("%s: No such host is known", NAME);
				break;
			case NO_RECOVERY:
				err_ret("%s: Unexpected server failure", NAME);
				break;
			case NO_DATA:
				err_ret("%s: No data available for this host", NAME);
				break;
		}		
		return NULL;
	}	

	memcpy(&in.s_addr, dns_data->h_addr_list[0], dns_data->h_length);
	return strdup(inet_ntoa(in));
}	
