#include <ctype.h>
#include <errno.h>
#include <netdb.h>		/* dns functions */
#include <setjmp.h>		/* setjmp, longjmp */
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <setjmp.h>
#include <syslog.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h> /* basic socket definitions */
#include <netinet/in.h> /* sockaddr_in{} and other Internet defns */
#include <sys/time.h>
#include <arpa/inet.h>  /* inet(3) functions */
#include <sys/utsname.h>/* uname stuff */
#include <time.h>			/* system time functions */
#include <unistd.h>
#include <assert.h>

#undef MAXLINE 
#define MAXLINE 10000

#define MIN_DELAY 30
#define MAX_DELAY 600

#define LOG_FACILITY LOG_USER
#define LOG_OPTION LOG_PID
#define LOG_PRIORITY LOG_INFO

#define NAME "yaunc"
#define DEFAULT_CONF_PATH "/etc/yaunc.conf"
#define OPTION_STRING "hf:"

#define UPHOST "www.uptimes.nu"
#define UPPORT 49153

/* program settings */
struct settings
{
	char * conf_file_path;
	char * auth;
	unsigned int delay;
	int daemon_flag;
	int already_daemon;
	int loops;
	int log_verbosity;
} settings;

/* global variables */
sigjmp_buf jumpbuffer;            /* for setjmp/longjmp */
struct settings settings;         /* holds all settings info */

/* general prototypes */
void usage(void);
int read_conf (void);
char * get_dns(const char *);
time_t get_uptime(float *);
char * encode(const char *, size_t);
int send_data(const char *);

/* signal handlers */
void sig_hup(int);

/* inlines */
void SetAuth (char *);
char * GetAuth (void);
void SetDelay (int);
int GetDelay (void);
void SetDaemonFlag (int);
int GetDaemonFlag (void);
void SetConfFilePath (char *);
char * GetConfFilePath (void);
void SetAlreadyDaemon (int);
int GetAlreadyDaemon (void);
void SetLoops (int);
int GetLoops (void);

/* prototypes for unp_error.c */
void err_ret(const char *, ...);
void err_sys(const char *, ...);
void err_dump(const char *, ...);
void err_msg(const char *, ...);
void err_quit(const char *, ...);
void err_log_ret(const char *, ...);
void err_log_msg(const char *, ...);
void err_log_dump(const char *, ...);
void err_log_sys(const char *, ...);

/* for signal handlers */
typedef  void  Sigfunc(int);

/* for networking code */
#define SA struct sockaddr
   /* in case it isn't defined */
#define socklen_t int

/* networking prototypes */
void Getpeername(int, struct sockaddr *, socklen_t *);
void Getsockname (int, struct sockaddr *, socklen_t *);
void Getsockopt(int, int, int, void *, socklen_t *);
int Select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
void Send(int, const void *, size_t, int);
void Sendmsg(int, const struct msghdr *, int);
int Socket(int, int, int );

/* other networking function */
int inet_pton(int, const char *, void *);
const char * inet_ntop(int, const void *, char *, size_t);
void Inet_pton(int, const char *, void *);

/* non-networking stuff */
int daemon_init(void);
